extern crate clap;

mod backtrack;

use std::time::SystemTime;

use clap::{load_yaml, App};

use backtrack::*;

fn main() {
    let yaml = load_yaml!("cli.yml");
    let matches = App::from_yaml(yaml).get_matches();

    let verbose = matches.is_present("verbose");

    let dim = matches.value_of("dimension");

    let series = matches.values_of("series").map(|v| {
        let values: Vec<_> = v.collect();
        values
    });

    if series.is_some() {
        let ser = series.unwrap();
        let min: usize = ser[0].parse().unwrap();
        let max: usize = ser[1].parse().unwrap();

        for n in min..max+1 {
            let start = SystemTime::now();

            let found = nreines_first_solution(n, verbose);

            let end = SystemTime::now();

            let duration = end.duration_since(start).unwrap();

            println!(
                "{:0>2.0} : {:>9} in {:<8}ms",
                n,
                if found { "found" } else { "not found" },
                duration.as_secs() * 1000 + duration.subsec_millis() as u64,
            );
        }
    } else if dim.is_some() {
        let n: usize = dim.unwrap().parse().unwrap();

        let start = SystemTime::now();

        let nb_solutions = nreines(n, verbose);

        let end = SystemTime::now();

        let duration = end.duration_since(start).unwrap();

        println!(
            "{} solutions trouvées en {:>8}ms",
            nb_solutions,
            duration.as_secs() * 1000 + duration.subsec_millis() as u64
        );
    }
}
